<?php



namespace Ron\DB;


class MsSqlDatabase extends Database
{

    /**
     * @var resource
     */
    private $connection;

    private $connectionString;
    private $dbName;

    public function __construct($connectionString, $dbName)
    {
        $this ->connectionString = $connectionString;
        $this ->dbName = $dbName;
    }

    /**
     * @return resource
     */
    public function getConnection()
    {
        return $this->connection;
    }


    /**
     * @return mixed
     */
    public function getConnectionString()
    {
        return $this->connectionString;
    }

    /**
     * @return mixed
     */
    public function getDbName()
    {
        return $this->dbName;
    }
    


    /**
     * @return string
     */
    public function getDbType()
    {
        return 'mssql';
    }

    /**
     * @param string $user
     * @param string $password
     * @throws DBException
     * @throws \Exception
     */
    public function connect($user, $password)
    {
        $this->connection = mssql_connect($this->connectionString, $user, $password);

        if (!$this ->connection) {
            throw new DBException($this, '', \Ron\ronText('Connection_fail'));
        }

        if ( !mssql_select_db($this->dbName,$this->connection) ) {
            throw new \Exception('Chyba pri volani funkce mssql_select_db - zkontrolujte jestli je spravne zadan nazev databaze!');
        }
    }


    /**
     * @return bool
     */
    public function close()
    {
        return mssql_close($this ->connection);
    }

    public function createParameters()
    {
        // driver mssql parametry neumi
        return new AdHocParameters($this);
    }


    /**
     * @param string $sqlText
     * @param Parameters $params
     * @return MsSqlCursor
     * @throws DBException
     */
    public function executeSelect($sqlText, $params = null)
    {
        $data = mssql_query($sqlText, $this->connection);

        if (!$data) {
            throw new DBException($this, $sqlText);
        };

        return new MsSqlCursor($this, $data);
    }

    /**
     * @param string $sqlText
     * @param Parameters $params
     * @throws DBException
     */
    public function executeSql($sqlText, $params = null)
    {
        $data = mssql_query($sqlText, $this->connection);

        if (!$data) {
            throw new DBException($this, $sqlText);
        };
    }
   
    /**
     * @return string
     */
    public function lastErrorText()
    {
        return mssql_get_last_message();
    }


    /**
     * @throws DBException
     */
    public function beginTransaction()
    {
        $this->executeSql('BEGIN TRAN');
    }

    /**
     * @throws DBException
     */
    public function commitTransaction()
    {
        $this->executeSql('COMMIT TRAN');
    }

    /**
     * @throws DBException
     */
    public function rollbackTransaction()
    {
        $this->executeSql('ROLLBACK TRAN');
    }



    /**
     * @param string $str
     * @return string
     */
    public function addSlashes($str)
    {
        return str_replace("'", "''", $str);
    }

    /**
     * @param string $text
     * @return string;
     */
    public function castWideMemo($text)
    {
        return "CAST($text AS TEXT)";
    }

    /**
     * @param string $text
     * @return string;
     */
    public function extractYear($text)
    {
        return "YEAR($text)";
    }

    /**
     * @param int|\DateTime $str
     * @return int|null
     */
    public function stringToTimestamp($str)
    {
        if ( !isset($str) ) return NULL;

           // datum z databaze muze mit ruzny format podle nastaveni v php.ini
           // Bud je v php.ini mssql.datetimeconvert = On (defaultni hodnota)
           //    a pak vraci datum ve formatu   5 V 2005 0:00
           // nebo je tam mssql.datetimeconvert = Off
           //    a pak vraci datum ve formatu   2005-05-01 00:00:00
           // a nebo jeste muze vracet
           //    may 5 2005 0:00 nebo tak nejak
           // V prvnim pripade lze primo puzit strtotime, v druhem pripade nee, ve tretim jo
           // Ve vsech pripadech ale windowsy nezvladaji prevod data mensiho nez 1.1.1970
        
        $str = str_replace('1899', '1970', $str); // pokud je tam 1899, tak je to nejspis jenom cas - prevede ho - finta, aby se cas dobre zobrazoval

        // Pokusi se prevest datum primo. Pokud se to nepovede tak funkce
        // vraci -1 a musi se to prechrchlat
        $result = strtotime($str);
        if ($result  &&  $result != -1)
        {
            return $result;
        }

        // Nepovedlo se prevest datum, protoze se vraci ve formatu
        // s rimskym cislem mesice
        $poma = explode (' ', $str);
        switch ($poma[1])
        {
            case "I" : $poma[1] = 'Jan'; break;
            case "II" : $poma[1] = 'Feb'; break;
            case "III" : $poma[1] = 'Mar'; break;
            case "IV" : $poma[1] = 'Apr'; break;
            case "V" : $poma[1] = 'May'; break;
            case "VI" : $poma[1] = 'Jun'; break;
            case "VII" : $poma[1] = 'Jul'; break;
            case "VIII" : $poma[1] = 'Aug'; break;
            case "IX" : $poma[1] = 'Sep'; break;
            case "X" : $poma[1] = 'Oct'; break;
            case "XI" : $poma[1] = 'Nov'; break;
            case "XII" : $poma[1] = 'Dec'; break;
        }

        $newstr = implode(" ", $poma);
        if (!$newstr) return NULL;
        $result = strtotime($newstr);
        return $result;
    }

    /**
     * @param int $timestamp
     * @return string
     */
    public function timestampToString($timestamp)
    {
        if ( !isset($timestamp) ) return 'null';

        if ($timestamp >= 31532400) return "{ ts '".date('Y-m-d H:i:s',$timestamp)."'}";
        //  $timestamp += 1609459200;    // rozdil mezi 1.1.1920 0:0:0 a 1.1.1971 0:0:0
        //  $rok = date("Y",$timestamp) - 51; // pocet let mezi 1920 a 1971
        $timestamp += 1262304000;    // rozdil mezi 1.1.2011 0:0:0 a 1.1.1971 0:0:0
        $rok = date("Y",$timestamp) - 40; // pocet let mezi 2011 a 1971
        return "{ ts '".$rok.date('-m-d H:i:s',$timestamp)."'}";
    }

    /**
     * @param int $timestamp
     * @return string
     */
    public function dateToString($timestamp)
    {
        if ( !isset($timestamp) ) return 'null';

        if ($timestamp >= 31532400) return "{ d '".date('Y-m-d',$timestamp)."'}";
        //  $timestamp += 1609459200;    // rozdil mezi 1.1.1920 0:0:0 a 1.1.1971 0:0:0
        //  $rok = date("Y",$timestamp) - 51; // pocet let mezi 1920 a 1971
        $timestamp += 1262304000;    // rozdil mezi 1.1.2011 0:0:0 a 1.1.1971 0:0:0
        $rok = date("Y",$timestamp) - 40; // pocet let mezi 2011 a 1971
        return "{ d '".$rok.date('-m-d',$timestamp)."'}";
    }

    /**
     * @param int $timestamp
     * @return string
     */
    public function timeToString($timestamp)
    {
        if ( $timestamp >= 0  &&  $timestamp < 86400)
        {
            // Jedna se pouze o cas. Kvuli letni/zimni to neprevadi pomoci date()
            $h = str_pad( (int)($timestamp / 3600), 2, '0', STR_PAD_LEFT);
            $timestamp = $timestamp - 3600 * $h;
            $m = str_pad((int)($timestamp / 60), 2, '0', STR_PAD_LEFT); 
            $s = str_pad($timestamp - 60 * $m, 2, '0', STR_PAD_LEFT);    
            return "{ ts '1899-12-30 $h:$m:$s'}";
        }   

        return "{ ts '1899-12-30 ".date('H:i:s', $timestamp)."'}";
    }

    public function timestampToParam($timestamp)
    {
        return new AdHocParameter($this ->timestampToString($timestamp));
    }

    public function dateToParam($timestamp)
    {
        return new AdHocParameter($this ->dateToString($timestamp));
    }

    public function timeToParam($timestamp)
    {
        return new AdHocParameter($this ->timeToString($timestamp));
    }


    function getBlobParam($data)
    {
        $datastring = unpack("H*hex", $data);
        return new AdHocParameter('0x' . $datastring['hex']);
    }


    public function stringToSecondsOfDay($str)
    {
        if ( !isset($str) ) return null;

        $timestr = strrchr($str, ' '); // vraci vse za posledni mezerou
        $poma = explode (":", $timestr);

        $add = ($poma[0] == 12)
            ? (stristr($timestr, 'am') === false ? 0 : -12)
            : (stristr($timestr, 'pm') === false ? 0 : 12);

        return ($poma[0] + $add) * 3600 + $poma[1] * 60.0 + $poma[2];
    }


    /**
     * prevede retezec z databaze na denni cas - vraci pocet hodin (realne cislo)
     * napr "... 7:30" => 7.5
     * @param string $str
     * @return integer
     */
    public function stringToTimeOfDay($str)
    {
        $val = $this ->stringToSecondsOfDay($str);
        return isset($val) ? $val / 3600 : null;
    }

}



class MsSqlCursor extends Cursor
{
    /**
     * @var resource
     */
    protected $data;


    /**
     * @param MsSqlDatabase $database
     * @param resource $data
     */
    function __construct($database, $data) {
        parent::__construct($database);
        $this ->data = $data;
    }


    public function close()
    {
        mssql_free_result($this ->data);
    }


    /**
     * @return resource
     */
    public function getData()
    {
        return $this->data;
    }


    /**
     * @return array|bool
     */
    public function fetch() {
        $rowData = mssql_fetch_row($this->data);
        return $rowData;
    }


    /**
     * Vraci pocet nactenych zaznamu
     * @return int
     */
    public function numRows() {
        return mssql_num_rows($this->data);        
    }


    /**
     * @param int $rowNumber
     * @return bool
     */
    public function seek($rowNumber)
    {
        return mssql_data_seek($this->data, $rowNumber);
    }

    /**
     * @return bool
     */
    public function last()
    {
        $cnt = mssql_num_rows($this->data);
        if ($cnt>0) return mssql_data_seek($this->data, $cnt-1);
        else return 0;        
    }

}





require_once('_appdbset.php');

function createDefaultDatabase() {
    global $db_connectionstring;
    global $db_name;

    $database = new MsSqlDatabase($db_connectionstring, $db_name);
    
    return $database;
}
