<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\DB;

use Exception;
use Ron\Profiler;

/**
 * Abstraktni proxy trida, ktera definuje pristup k databazi.
 * Tridy odvozene od teto tridy pak implementuji pristup do konkretnich databazi (SQL Server, Oracle, Firebird atd.)
 */
abstract class Database
{
    /** @var string     Operator, ktery se v SQL vyrazech pouziva pro spojovani retezcu */
    public $concatenateOperator = '+';

    /** @var string[] */
    public $functions = [];


    /**
     * Vraci identifikator typu databaze.
     * @return string
     */
    abstract public function getDbType();

    /**
     * Pripoji se k databazi se zadanymi prihlasovacimi udaji
     * @param string $user
     * @param string $password
     * @throws DBException
     */
    abstract public function connect($user, $password);

    /**
     * Ukonci pripojeni k databazi
     * @return bool
     */
    abstract public function close();

    /**
     * Vraci objekt pro nastaveni parametru SQL dotazu - interface Parameters
     * @return Parameters
     */
    abstract public function createParameters();


    /**
     * Nahradi zadany alias funkce ($functionName) v SQL dotazu skutecnou funkci pouzivanou v pripojene databazi
     * @param $functionName
     * @param $argument
     * @return string
     */
    public function replaceFunction($functionName, $argument)
    {
        $name = strtoupper($functionName);
        switch ($name) {
            case 'CASTWIDEMEMO' :
                return $this ->castWideMemo($argument);

            case 'CONCAT' :
                return $this ->concatenateOperator;

            case 'YEAR' :
                return $this->extractYear($argument);

            default :
                $func = $this->functions[$name] ?? $functionName;
        }

        if ($func  &&  !is_string($func)  &&  is_callable($func)) {
            return $func($argument);
        }

        if ($func  &&  $func[0] == '#') {
            return substr($func, 1);    // bez argumentu
        }

        if ($func  &&  strpos($func, '?') !== false) {
            return str_replace('?', $argument, $func);
        }

        return $func
            ? $func . '(' . $argument . ')'
            : $argument;
    }

    /**
     * Nahradi "Makra" pro funkce {fn_...(...)} volanim prislusne funkce
     * @param string $sqlText
     * @param Parameters $params
     * @return string
     */
    function processSQLText($sqlText, /** @noinspection PhpUnusedParameterInspection */ $params)
    {
        $result = '';

        $lastPos = 0;
        $pos = 0;
        do {
            $pos = strpos($sqlText, '{fn_', $pos);
            if ($pos !== false) {
                $result .= substr($sqlText, $lastPos, $pos - $lastPos);
                // najde ( - za ni zacina parametr
                $leftBrPos = strpos($sqlText, '(', $pos + 4);
                $functionName = $leftBrPos ? trim(substr($sqlText, $pos + 4, $leftBrPos - $pos - 4)) : null;
                $rightBrPos = $leftBrPos ? strpos($sqlText, ')}', $leftBrPos + 1) : null;

                if ($functionName && $leftBrPos && $rightBrPos) {
                    // nasel funkci, levou i pravou zavorku
                    $argument = substr($sqlText, $leftBrPos + 1, $rightBrPos - $leftBrPos - 1);

                    $result .= $this ->replaceFunction($functionName, $argument);
                    $pos = $rightBrPos + 2; // posune se za )}
                }
            } else {
                $result .= substr($sqlText, $lastPos);
            }
            $lastPos = $pos;
        } while ($pos !== false);

        return $result;
    }


    /**
     * Spusti zadany SQL vyraz a vraci object Cursor
     * @param string $sqlText
     * @param Parameters $params
     * @return Cursor
     * @throws DBException
     */
    abstract protected function executeSelect($sqlText, $params = null);


    /**
     * Spusti zadany SQL SELECT vyraz. Vraci kurzor.
     * @param SqlStatement|string $sqlText SQL vyraz, ktery se ma provest
     * @param Parameters $params          Parametry pro SQL vyraz. Pouzije se pouze v pripade, ze je prvni parametr string!
     * @return Cursor
     * @throws DBException
     */
    public function select($sqlText, $params = null)
    {
        if ($sqlText instanceof SqlStatement) {
            if ($params) {
                throw new DBException($this, $sqlText, 'Chyba pri volani Database::select - params musi byt null!');
            }
            $params = $sqlText ->parameters;
            $sqlText = $sqlText ->sqlText;
        }

        $sqlText = $this ->processSQLText($sqlText, $params);
        $profilerItem = Profiler::addItem($sqlText, 'SQL SELECT');
        if ($profilerItem  &&  $params) {
            $text = $params ->getLogText();
            if ($text) {
                $profilerItem->texts[] = $text;
            }
        }

        try {
            $result = $this->executeSelect($sqlText, $params);
            Profiler::endItem($profilerItem);
        } catch (Exception $e) {
            if ($profilerItem) {
                $profilerItem ->texts[] = 'ERROR: ' . $e ->getMessage() . "\n";
            }
            Profiler::endItem($profilerItem);

            if ($e instanceof DBException) {
                throw $e;
            }

            throw new DBException($this, $sqlText, null, $e, $e ->getMessage()); // , $e); // $e ->getMessage()
        }

        return $result;
    }

    /**
     * Spusti zadany SQL vyraz (INSERT, UPDATE atd.)
     * @param SqlStatement|string $sqlText  SQL vyraz, ktery se ma provest
     * @param Parameters $params            Parametry pro SQL vyraz. Pouzije se pouze v pripade, ze je prvni parametry string!
     * @throws DBException
     */
    abstract protected function executeSql($sqlText, $params = null);

    /**
     * Spusti zadany SQL prikaz (INSERT, UPDATE, ...).
     * @param SqlStatement|string $sqlText
     * @param Parameters $params  Pouzije se pouze v pripade, ze je prvni parametr string!
     * @returns void    Poznamka - return je deprecated - jen pro stary kod, ktery testoval navratovou hodnotu
     * @throws DBException
     */
    public function sql($sqlText, $params = null)
    {
        if ($sqlText instanceof SqlStatement) {
            if ($params) {
                throw new DBException($this, $sqlText ->sqlText, 'Chyba pri volani Database::sql - params musi byt null!');
            }
            $params = $sqlText ->parameters;
            $sqlText = $sqlText ->sqlText;
        }

        $sqlText = $this ->processSQLText($sqlText, $params);
        $profilerItem = Profiler::addItem($sqlText, 'SQL COMMAND');
        if ($profilerItem  &&  $params) {
            $text = $params ->getLogText();
            if ($text) {
                $profilerItem->texts[] = $text;
            }
        }

        try {
            $this->executeSql($sqlText, $params);
        } catch (DBException $e) {
            if ($profilerItem) {
                $profilerItem ->texts[] = '    CHYBA ' . $e ->getMessage();
                Profiler::endItem($profilerItem);
            }

            throw $e;
        } catch (Exception $e) {
            if ($profilerItem) {
                $profilerItem ->texts[] = '    CHYBA ' . $e ->getMessage();
                Profiler::endItem($profilerItem);
            }

            throw new DBException($this, $sqlText, null, $e, $e ->getMessage()); // , $e); // $e ->getMessage()
        }

        Profiler::endItem($profilerItem);

        return true; // Pro kompatibilitu se starym kodem, ktery testuje vysledek. Nyni se pri chybe vyhazuje vyjimka.
    }


    /**
     * Vraci SQL vyraz pro zjisteni nove hodnoty pro zadanou sekvenci. Lze pouzit samostatne pro zjisteni hodnoty, nebo primo v SQL vyrazu pro vkladani
     * @param $sequenceName
     * @return string
     */
    public function nextSequenceValueSQL($sequenceName)
    {
        return "NEXT VALUE FOR $sequenceName";
    }


    /**
     * Zjisti, jestli v databazi existuje sekvence zadaneho jmena.
     * @param string $sequenceName
     * @return bool
     */
    protected function sequenceExistsInternal(string $sequenceName): bool
    {
        return false;
    }

    /**
     * Asociativni pole (cache) jiz overovanych sekvenci
     * @var bool[]
     */
    private $sequences = [];


    /**
     * Vraci true, pokud existuje sekvence zadaneho jmena. Pro dalsi pouziti si ulozi vysledek do cache $sequences
     * @param string $sequenceName
     * @return bool
     */
    public function sequenceExists(string $sequenceName): bool
    {
        if (!isset($this ->sequences[$sequenceName])) {
            $this ->sequences[$sequenceName] = $this ->sequenceExistsInternal($sequenceName);
        }

        return $this ->sequences[$sequenceName];
    }


    /**
     * @return mixed
     * @throws Exception
     */
    public function lastAutoincrementValue()
    {
        throw new Exception(get_class($this) . ' error - method lastAutoincrementValue is not implemented!');
    }

    /**
     * Vraci posledni chybovou hlasku z databaze. Presna funkcnost zalezi na implementaci v databazovem driveru.
     * @return string
     */
    abstract public function lastErrorText();


    /**
     * Spusti novou SQL transakci.
     * @throws DBException
     */
    abstract public function beginTransaction();

    /**
     * Potvrdi SQL transakci.
     * @throws DBException
     */
    abstract public function commitTransaction();

    /**
     * Ukonci SQL transakci a vrati zpet provedene zmeny.
     * @throws DBException
     */
    abstract public function rollbackTransaction();

    /**
     * Upravi zadany retezec tak, aby se dal pouzit v SQL dotazu - napriklad muze nahradit apostrofy.
     * @param string $str
     * @return string
     * @deprecated
     */
    abstract public function addSlashes($str);

    /**
     * Upravi zadany text, aby sel zapsat do datoveho pole typu "MEMO" (vyzaduji to nektere drivery s nekterymi verzemi databaze)
     * @param string $text
     * @return string
     */
    abstract public function castWideMemo($text);

    /**
     * Vraci SQL vyraz, ktery ze zadaneho argumentu (napriklad jmeno datoveho pole) extrahuje rok
     * @param string $text
     * @return string
     */
    abstract public function extractYear($text);

    /**
     * Prevede data vracena z databazoveho pole typu timestamp (podle implementace DB driveru to byva retezec nebo objekt) na timestamp
     * @param string $str
     * @return int|null
     */
    abstract public function stringToTimestamp($str);

    /**
     * Prevede zadanou hodnotu (timestamp) na SQL konstantu
     * @param int $timestamp
     * @return string
     * @deprecated
     */
    abstract public function timestampToString($timestamp);

    /**
     * Prevede zadanou hodnotu (timestamp) na sql konstantu. Pokud obsahuje i cas, tak se tento cas ignoruje (tj. vzdy je vysledek s casem 0:00:00)
     * @param int $timestamp
     * @return string
     * @deprecated
     */
    abstract public function dateToString($timestamp);

    /**
     * Prevede zadany cas (timestamp) na SQL konstantu.
     * @param int $timestamp
     * @return string
     * @deprecated
     */
    abstract public function timeToString($timestamp);

    /**
     * Vytvori parametr ze zadane hodnoty.
     * @param int $timestamp
     * @return mixed
     */
    abstract public function timestampToParam($timestamp);

    /**
     * Vytvori parametr ze zadane hodnoty. Pouzije pouze datum, cas ignoruje.
     * @param int $timestamp
     * @return mixed
     */
    abstract public function dateToParam($timestamp);

    /**
     * Vytvori parametr ze zadane hodnoty. Pouzije pouze cas, datum ignoruje.
     * @param int $timestamp
     * @return mixed
     */
    abstract public function timeToParam($timestamp);


    /**
     * Prevede retezec z databaze na denni cas - vraci pocet hodin (realne cislo)
     * napr "... 7:30" => 7.5
     * @param string $str
     * @return float
     */
    abstract public function stringToTimeOfDay($str);

    /**
     * prevede retezec z databaze na denni cas - vraci pocet sekund od pulnoci
     * @param string $str
     * @return integer|null
     */
    public function stringToSecondsOfDay($str)
    {
        $val = $this ->stringToTimeOfDay($str);
        return isset($val)
            ? (int)($val * 3600)
            : null;
    }


    /**
     * Vraci vysledek zadaneho sql dotazu. Pri chybe vyhodi vyjimku.
     * @param string|SqlStatement   $sql
     * @param Parameters    $params
     * @param string        $errorMessage
     * @return Cursor
     * @throws DBException
     */
    function getSqlResult($sql, $params = null, $errorMessage = null)
    {
        // pro kompatibilitu starsiho kodu (druhy parametr byl errorMessage)
        if (is_string($params)) {
            if (!$errorMessage) {
                $errorMessage = $params;
            }
            $params = null;
        }

//        try {
            $cursor = $this->select($sql, $params);
//        } catch (DBException $e) {
//            throw new DBException($this, $sql, $errorMessage, $e ->getPrevious(), $e ->dbMessage);
//        }

        return $cursor;
    }


    /**
     * Zpracuje hodnotu nactenou z blob pole
     * @param $value
     * @return mixed
     */
    function stringToBinary($value)
    {
        return $value;
    }


    /**
     * Vraci hodnotu parametr pro pole typu BLOB
     * @param mixed $data
     * @param string $fieldName  Jmeno pole, do ktereho se hodnota uklada - je potreba napr. na Oracle
     * @return mixed
     */
    function getBlobParam($data, $fieldName)
    {
        return $data;
    }
}
