<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace Ron\DB;

use Ron\StringUtils;

/**
 * Definuje pomocne funkce pro praci s SQL dotazy
 */
class SQLUtils
{

    /**
     * Sestavi dotaz ze zadanych casti.
     * wherePart a orderPart mohou byt prazdne
     * @param string|string[] $selectPart       Cast SQL dotazu, ktera se vlozi za SELECT (seznam poli jako SQL vyraz nebo jako pole). Pokud je prazdne, vlozi se do vyrazu '*'
     * @param string|SqlStatement $fromPart     Cast SQL dotazu za FROM - napriklad jmeno tabulky, nebo vyraz obsahujici spojeni vice tabulek
     * @param string|SqlStatement $wherePart    Podminka SQL dotazu. Nemusi byt zadana
     * @param string $orderPart                 ORDER BY cast SQL dotazu. Nemusi byt zadana
     * @param bool $distinct                    Urcuje, jestli se ma do dotazu vlozit DISTINCT
     * @param string $groupByPart
     * @return string|SqlStatement
     */
    static function createSql($selectPart, $fromPart, $wherePart = null, $orderPart = null, $distinct = false, $groupByPart = '')
    {
        $params = null;

        // vytvori select cast dotazu
        $sqlText = 'SELECT ' . ($distinct ? 'DISTINCT ' : '');

        if (isset($selectPart)) {
            if (is_array($selectPart)) {
                $sqlText = $sqlText . $selectPart[0];
                for ($i = 1; $i < count($selectPart); $i++) {
                    $sqlText = $sqlText . ', ' . $selectPart[$i];
                }
            } else if (is_string($selectPart)) {
                $sqlText = $sqlText . $selectPart;
            } else {
                $sqlText = $sqlText . '*';
            }
        } else {
            $sqlText = $sqlText . '*';
        }

        // prida from cast dotazu
        if ($fromPart instanceof SqlStatement) {
            $sqlText .= "\nFROM " . $fromPart ->sqlText;

            $params = $fromPart ->parameters ->newInstance();
            $params ->appendParameters($fromPart ->parameters);
        } else {
            $sqlText = $sqlText . "\nFROM " . $fromPart;
        }

        // prida filtr pokud je zadan
        if (isset($wherePart)) {
            if ($wherePart instanceof SqlStatement) {
                if ($wherePart ->sqlText) {
                    if ($params) {
                        $params ->appendParameters($wherePart->parameters);
                    } else {
                        $params = $wherePart->parameters;
                    }
                    $sqlText .= "\nWHERE " . $wherePart ->sqlText;
                }
            } else if ($wherePart != '') {
                $sqlText = $sqlText . "\nWHERE " . $wherePart;
            }
        }

        // prida group by
        if (isset($groupByPart)  &&  $groupByPart != '') {
            $sqlText = $sqlText . "\nGROUP BY " . $groupByPart;
        }

        // prida razeni
        if (isset($orderPart)  &&  $orderPart !== '') {
            $sqlText = $sqlText . "\nORDER BY " . $orderPart;
        }

        if ($params) {
            return new SqlStatement($sqlText, $params);
        }

        return $sqlText;
    }


    /**
     * Sestavi IN podminku ze zadanych hodnot
     * @param Database  $database
     * @param string    $fieldName      Jmeno databazoveho pole nebo SQL vyraz
     * @param string|string[]|int|int[] $values Seznam hodnot pro IN podminku
     * @param Parameters|bool $params   Objekt parametru, nebo true pokud se ma pouzit addSlashes nebo false, pokud se ma vlozit primo zadana hodnota
     * @return string
     */
    static function getINCondition($database, $fieldName, $values, $params = true)
    {
        $result = $fieldName;
        $useParams = $params instanceof Parameters;

        if (is_array($values)) {
            $result .= ' IN (';
            $first = true;

            foreach ($values as $value) {
                if (!$first) {
                    $result .= ', ';
                } else {
                    $first = false;
                }

                if ($useParams) {
                    $result .= $params ->addParameter($value);
                } else if ($params) {
                    $result .= "'" . $database->addSlashes($value) . "'";
                } else {
                    $result .= $value;
                }
            }

            $result .= ')';
        } else {
            $result .= ' = ';

            if ($useParams) {
                $result .= $params->addParameter($values);
            } else if ($params) {
                $result .= "'" . $database->addSlashes($values) . "'";
            } else {
                $result .= $values;
            }
        }

        return $result;
    }


    /**
     * Spoji zadane WHERE casti SQL vyrazu (stringy nebo instance SqlStatement) logickou spojkou AND.
     * Vraci vysledny vyraz - pokud jsou v $parts jen stringy, vraci string, jinak vraci SqlStatement.
     * Jednotlive casti mohou byt i prazdne.
     * @param (string|SqlStatement)[] $parts
     * @return string|SqlStatement
     */
    static function concatenateWherePartsOfSql($parts)
    {
        $result = '';

        foreach ($parts as $part) {
            if (isset($part)) {

                if ($part instanceof SqlStatement) {
                    if ($part ->sqlText != '') {
                        if (!$result instanceof SqlStatement) {
                            // $result je zatim retezec - udela z nej SqlFilter
                            $result = new SqlStatement($result, $part->parameters->newInstance());
                        }

                        $result ->sqlText .= ($result ->sqlText ? ' AND ' : '') . '(' . $part ->sqlText . ')';
                        $result ->parameters ->appendParameters($part ->parameters);
                    }

                } else if ($part != '') {
                    // pripoji retezec
                    if ($result instanceof SqlStatement) {
                        $result ->sqlText .= ($result ->sqlText ? ' AND ' : '') . '(' . $part . ')';
                    } else {
                        $result .= ($result ? ' AND ' : '') .  '(' . $part . ')';
                    }
                }
            }
        }

        return $result;
    }


    /**
     * Vraci nove seriove cislo zaznamu - vyhleda maximalni existujici hodnotu a vraci hodnotu o 1 vetsi, nebo 1 v pripade, ze nic nenajde
     * @param Database $database
     * @param string $tableName     Jmeno tabulky
     * @param string $fieldName     Jmeno databazoveho pole (napr. 'Id')
     * @param SqlStatement|string|null $filter  Nepovinny filtr - pokud je zadan, tak se maximalni existujici hodnota vyhledava jen v zaznamech odpovidajicich podmince
     * @param Parameters|null $filterParams     Hodnoty parametru pro $filtr
     * @return int
     * @throws DBException
     */
    static function newSerialNumber($database, $tableName, $fieldName, $filter = null, $filterParams = null)
    {
        $result = 1;

        $sql = "SELECT MAX($fieldName) FROM $tableName";
        if ($filter) {
            if ($filter instanceof SqlStatement) {
                $filterParams = $filter ->parameters;
                $sql .= ' WHERE ' . $filter ->sqlText;
            } else {
                $sql .= ' WHERE ' . $filter;
            }
        }

        $cursor = $database->getSqlResult($sql, $filterParams);

        if ($row = $cursor->fetch()) {
            $result = $row[0] + 1;
        }

        $cursor ->close();

        return $result;
    }


    /**
     * Vraci novou hodnotu pro zadanou sekvenci
     * @param Database $database
     * @param string $sequenceName  Jmeno sequence
     * @return int
     * @throws DBException
     */
    public static function nextSequenceValue($database, $sequenceName)
    {
        $cursor = $database ->getSqlResult(
            'SELECT ' . $database ->nextSequenceValueSQL($sequenceName) .
            ($database ->getDbType() === 'oracle' ? ' FROM DUAL' : '')
        );
        $row = $cursor ->fetch();
        $cursor ->close();

        return (int)$row[0];
    }



    /**
     * Ze zadaneho SQL vyrazu zjisti jmeno tabulky
     * @param string $sql
     * @return string
     */
    static function getTableNameFromSQL($sql)
    {
        if (preg_match('/^\s*(\w+)\s*(\w+)\s*(\w+)/', $sql, $pomarray)) {
            // prvni, druhe a treti slovo
            if (!$pomarray[1]) return '';

            if (!strcasecmp('INSERT', $pomarray[1])) {
                // podle gfiles
                if (!strcasecmp('INTO', $pomarray[2])) {
                    return $pomarray[3];
                } else {
                    return $pomarray[2];
                }
            }

            if (!strcasecmp('UPDATE', $pomarray[1])) {
                return $pomarray[2];
            }

            if (!strcasecmp('DELETE', $pomarray[1])) {
                return $pomarray[3];
            }
        }

        // u selectu jmeno tabulky nevyhledava

        return '';
    }


    /**
     * Ze zadaneho SQL vyrazu zjisti typ (insert/update/delete/select)
     * @param string $sql
     * @return string
     */
    static function getEventTypeFromSQL($sql)
    {
        if ( preg_match('/^\s*(\w+)/', $sql, $pomarray) ) {
            // prvni slovo za whitespace
            if ( !$pomarray[1] ) {
                return '';
            }

            if ( !strcasecmp('INSERT', $pomarray[1]) ) {
                return 'INSERT';
            }

            if ( !strcasecmp('UPDATE', $pomarray[1]) ) {
                return 'UPDATE';
            }

            if ( !strcasecmp('DELETE', $pomarray[1]) ) {
                return 'DELETE';
            }

            if ( !strcasecmp('SELECT', $pomarray[1]) ) {
                return 'SELECT';
            }

            if ( !strcasecmp('CREATE', $pomarray[1]) ) {
                return 'CREATE';
            }

            if ( !strcasecmp('ALTER', $pomarray[1]) ) {
                return 'ALTER';
            }
        }

        return '';
    }


    /**
     * @param Database $database
     * @param $tableName
     * @param $serialCodeFieldName
     * @param string $mask
     * @param int $serialCodeMin
     * @param int $serialCodeMax
     * @return bool|string
     * @throws \Exception
     */
    static function getNextRecordSerialCode($database, $tableName, $serialCodeFieldName, $mask, $serialCodeMin, $serialCodeMax)
    {
        //Vytvori min a max dle masky a dle tohoto bude filtrovat
        $min = StringUtils::assemblyStringIDFromValueAndMask($mask, $serialCodeMin);
        $max = StringUtils::assemblyStringIDFromValueAndMask($mask, $serialCodeMax);

        //Delka retezce pro podminku
        $lengthMask = strlen($mask);

        //Nacteni posledni(max) hodnotu v dane tabulce pro potreby vygenerovani nasledujici
        $params = $database->createParameters();
        $sql =
            "SELECT Max($serialCodeFieldName) AS cislo FROM $tableName" .
            " WHERE $serialCodeFieldName >= " . $params->addParameter($min). " AND $serialCodeFieldName <= " . $params->addParameter($max) .
            " AND {fn_LEN($serialCodeFieldName)} = " .$params->addParameter($lengthMask);

//        if ($filtr != ""){
//            $sql .= " AND (" . $filtr . ")";
//        }

        $cursor = $database->getSqlResult($sql, $params);
        $lastSerialCode = $cursor->fetch()[0];

        if ($lastSerialCode == $max){
            throw new \Exception('dosazeno_max_ID');
        }

        //Pokud neexistuje predchozi v rozsahu, pak nastav na prvni hodnotu
        if ($lastSerialCode == ""){
            $newSerialCode = StringUtils::assemblyStringIDFromValueAndMask($mask, $serialCodeMin);
        } else {
            //Pokud existuje predchozi v rozsahu, pak vytvori nasledujici hodnotu v rozsahu
            $newSerialCode = StringUtils::assemblyStringIDFromValueAndMask($mask, StringUtils::getNumberOrRangeFromStringID($mask, $lastSerialCode) , true);
        }
        $cursor->close();

        return $newSerialCode;
    }

    /** Zkontroluje zda vkladany zaznam zaznam ($record) existuje v nadrazene tabulce ($table).
     * @param Database $database
     * @param string $record
     * @param string $table
     * @param string $primaryFieldName
     * @return bool vraci true, pokud nadrazeny zaznam existuje
     * @throws \Exception
     */
    public static function existsMasterRecord($database, $record, $table, $primaryFieldName)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT $primaryFieldName FROM $table WHERE $primaryFieldName = " . $params->addParameter($record);

        return $record == DBUtils::getSqlValue($database, $sql, $params);
    }

}
