<?php
/**
 * stanislav.elbl, 11.06.2021 16:15
 */

namespace RonPortal\Dashboard;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\JsonController;
use Ron\MVC\Model;
use Ron\MVC\TextViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NastaveniUzivatele;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalXMLEvents;
use RonPortal\User;
use RonPortal\XML\RonPortalXMLForm;

class DashboardController extends JsonController
{
    /** @var Database */
    var $database;


    /**
     * @param Database $database
     */
    public function __construct($database)
    {
        $this ->database = $database;
    }


    /**
     * Doplni preddefinovane panely podle z adresare dashboard
     * @param DashboardFrames $dashboardFrames
     * @throws \Exception
     */
    private function addDefaultFrames($dashboardFrames)
    {
        $dirName = 'dashboard';
        $dh = opendir($dirName);

        $fileList = [];
        while (false !== ($fileName = readdir($dh)))
            if (!is_dir($dirName . '/' . $fileName)) {
                $fileList[] = $fileName;
            }

        foreach ($fileList as $fileName) {
            /** @noinspection PhpIncludeInspection */
            $newFrame = require($dirName . '/' . $fileName);

            if ($newFrame) {
                if (!$newFrame instanceof DashboardFrame) {
                    throw new \Exception("Chyba - neplatny soubor ve slozce dashboard ($fileName)!");
                }

                if ((strlen($newFrame ->id) > 20)  &&  Prava::vratPrava('admin') ->jsouNejakePrava()) {
                    MessageManager::addWarning("Kod dashboardu {$newFrame ->id} je prilis dlouhy. Nebude mozne vytvorit uzivatelske pravo pro jeho zobrazeni!");
                }

                $dashboardFrames->frames[] = $newFrame;
            }
        }
    }


    /**
     * Doplni XML panely DASHBOARD_....
     * @param DashboardFrames $dashboardFrames
     * @throws DBException
     */
    private function addXmlFrames($dashboardFrames)
    {
        $sql =
            'SELECT Id, Popis, TypPrava FROM SablonaXMLFormulare ' .
            " WHERE Popis LIKE 'DASHBOARD%'" .
            ' ORDER BY Popis';

        $cursor = $this->database->getSqlResult($sql, null, 'Chyba pri nacitani XML formularu!');

        while ($row = $cursor->fetch()) {
            $dashboardFrames ->addXmlFrame($row[0], trim(substr($row[1], 9)), $row[2]);
        }

        $cursor ->close();
    }


    /**
     * @param bool $sort    Priznak, jestli se maji panely seradit podle nastaveni uzivatele
     * @return DashboardFrames
     */
    function initDashboards($sort = false)
    {
        $result = new DashboardFrames();

        $this ->addDefaultFrames($result);

        try {
            $this->addXmlFrames($result);
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        // seradi panely podle uzivatele
        if ($sort) {
            $userOrderStr = NastaveniUzivatele::getUserSettingsItem($this ->database, 'dashboardframes');
            if ($userOrderStr) {
                $result ->sort($userOrderStr);
            }

            $userSettingsStr = NastaveniUzivatele::getUserSettingsItem($this ->database, 'dashboardsettings');
            if ($userSettingsStr) {
                $result ->loadSettings($userSettingsStr);
            }
        }

        return $result;
    }


    /**
     * @param DashboardFrame $frame
     * @return string
     */
    function loadSql($frame)
    {
        if ($frame ->prava) {
            if (!Prava::jsouNejakePrava($frame ->prava)) {
                return \Ron\ronText('Nemate_pravo');
            }
        }

        $sql = $frame ->sql;
        $sql = str_replace('{$RC}', "'" . $this ->database ->addSlashes(User::getRC()) . "'", $sql);

        if (!$sql) return 'Chyba pri nacitani dat - neni zadan SQL dotaz'; // nemelo by nastat
        $cursor = $this ->database ->getSqlResult($sql);
        $class = 'rowOver';
        if ($frame ->linkUrl) {
            $class .= ' clickHref clickLink';
        }

        $result = "<table class='$class'>";
        while ($row = $cursor ->fetch()) {
            $result .= '<tr>';
            foreach ($row as $col) {
                $result .= '<td>' . StringUtils::htmlTextConvert($col) . '</td>';
            }
            $result .= '</tr>';
        }
        $cursor ->close();
        return $result;
    }


    function getXmlHtml($xmlId)
    {
        $form = new RonPortalXMLForm($this ->database);
        $form -> nactiXMLFormular($xmlId, true); // zeSablony = true

        // kontrola prav
        if ($form ->typPrava) {
            if (!Prava::jsouNejakePrava($form ->typPrava)) {
                return \Ron\ronText('Nemate_pravo');
            }
        }

        $form ->loadXML();

        if ($form ->mode != 'show') {
            throw new \Exception('Mod XML formulare musi byt "show"!');
        }

        if ($form ->passwordProtected  &&  !RonPortalSession::enablePasswordProtected()) {
            return 'Chraneno heslem ...';
        }

        return $form ->getHtmlOutput();
    }


    function getHtml()
    {
        $frames = $this ->initDashboards();
        if (isset($_GET['id'])) {
            // sql
            $frame = $frames->findFrame($_GET['id']);
            if (!$frame) {
                return StringUtils::fromUTF8('<div>Chyba - neplatné ID</div>');
            }

            if ($frame ->sql) {
                return $this ->loadSql($frame);
            }
        } else if (isset($_GET['xmlid'])) {
            // xml
            $frame = $frames->findXmlFrame($_GET['xmlid']);
            if (!$frame) {
                return StringUtils::fromUTF8('<div>Chyba - neplatné xmlID</div>');
            }

            return $this ->getXmlHtml($frame ->xmlId);
        } else {
            // chyba
            return '<div>Chyba - neni zadano id panelu</div>';
        }

        return 'Spatne nakonfigurovany frame - napriklad neni zadan SQL dotaz'; // spatne nakonfigurovany frame
    }


    /**
     * Uzivatel prehodil poradi panelu - ulozi
     * @param $newOrder
     * @throws \Exception
     */
    function sortFrames($newOrder)
    {
        NastaveniUzivatele::setUserSettingsItem($this ->database, 'dashboardframes', $newOrder);
    }


    private function setFrameSize($frameId, $newSizeStr)
    {
        $old = NastaveniUzivatele::getUserSettingsItem($this ->database, 'dashboardsettings');
        $setArray = explode(';', $old);

        $found = false;
        $new = '';
        foreach ($setArray as $frameSet) {
            //list($id, $sizeStr, $color) = explode(':', $frameSet);
            list($id, $sizeStr) = explode(':', $frameSet);

            if ($id == $frameId) {
                $found = true;
                $new .= ($new ? ';' : '') . $id . ':' . $newSizeStr; // . ($color ?: '');
            } else {
                $new .= ($new ? ';' : '') . $frameSet;
            }
        }

        if (!$found) {
            $new .= ($new ? ';' : '') . $frameId . ':' . $newSizeStr;
        }

        NastaveniUzivatele::setUserSettingsItem($this ->database, 'dashboardsettings', $new);
    }


    function resizeFrame()
    {
        //$width = (int)$_GET['width'];
        $height = (int)$_GET['height'];
        $cols = (int)$_GET['cols'];

        if ($cols > 0  &&  $height > 0) {
            $this->setFrameSize($_GET['id'], $cols . 'x' . $height);
        }
    }


    function hideFrame($frameId)
    {
        if (!$frameId) {
            return;
        }

        $this->setFrameSize($frameId, 'off');

        $visibleFrames = NastaveniUzivatele::getUserSettingsItem($this ->database, 'dashboardframes');
        NastaveniUzivatele::setUserSettingsItem($this ->database, 'dashboardframes', StringUtils::removeSubstring(';', $visibleFrames, $frameId));
    }


    function showFrame($frameId)
    {
        if (!$frameId) {
            return;
        }

        $this->setFrameSize($frameId, 'def');

//        $visibleFrames = NastaveniUzivatele::getUserSettingsItem($this ->database, 'dashboardframes');
//        NastaveniUzivatele::setUserSettingsItem($this ->database, 'dashboardframes', StringUtils::removeSubstring(';', $visibleFrames, $frameId));
    }


    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'loaddata' :
                $output ->addText('html', $this ->getHtml());
                break;

            case 'sort' :
                $this ->sortFrames($_GET['order']);
                break;

            case 'resize' :
                $this ->resizeFrame();
                break;

            case 'hide' :
                $this ->hideFrame($_GET['id']);
                break;

            case 'show' :
                $this ->showFrame($_GET['id']);
                break;
        }
    }


    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('dashboard.ini');

        Prava::nastavObdobiProPrava(time());

        if (!Prava::jsouNejakePrava('dashboard')) {
            throw new NeniPravoException("Dashboard neni povolen v pravech uzivatele!");
        }

        if (isset($_GET['action'])) {
            switch ($_GET['action']) {
                case 'translations' :
                    return $this ->translations();

                default :
                    return parent::handleAction($action);
            }
        }

        $this ->addTranslationsScript('dashboard');

        Model::getInstance()
            ->addScript('script/dashboard.js', 'dashboard-js')
            ->addScript('script/chart.js', 'chart-js')
            ->addScript('script/chartjs-plugin-datalabels.min.js', 'chartjs-plugin-datalabels-js');

        // div pro kazdou agendu

        $bannerHtml = RonPortalXMLForm::xmlEvents($this ->database, RonPortalXMLEvents::DASHBANER, null, '', true);

        if ($bannerHtml) {
            Model::addViewObject(new TextViewObject($bannerHtml));
        }

        $result = new DashboardView($this ->database, $this ->initDashboards(true));

        return $result;
    }
}
