<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal;

use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\SQLUtils;
use Ron\GuiUtils;
use Ron\StringUtils;

class PoznamkaKeDni
{
    public $id;

    public $datumOd;
    public $datumDo;
    public $typ;
    public $barva;
    public $text;
    public $poznamka;

    /** @var int */
    public $kodTypuRozdeleni;
    /** @var string */
    public $rozdeleni;

    function getJson()
    {
        return [
            'id' => $this ->id,
            'datumOd' => StringUtils::jsonDate($this ->datumOd),
            'datumDo' => StringUtils::jsonDate($this ->datumDo),
            'typ' => StringUtils::jsonText($this ->typ),
            'barva' => isset($this ->barva) ? GuiUtils::getHtmlColor($this ->barva) : null,
            'text' => StringUtils::jsonText($this ->text),
            'poznamka' => StringUtils::jsonText($this ->poznamka),
            'kodTypuRozdeleni' => $this ->kodTypuRozdeleni,
            'rozdeleni' => isset($this ->rozdeleni) ? StringUtils::jsonText($this ->rozdeleni) : null
        ];
    }


    /**
     * @param Database $database
     * @param string $sql
     * @param Parameters $params
     * @return PoznamkaKeDni[]
     */
    private static function nactiPoznamkySQL($database, $sql, $params)
    {
        $cursor = $database ->getSqlResult($sql, $params);

        $poznamky = [];

        while ($row = $cursor ->fetch()) {
            $poznamka = new PoznamkaKeDni();

            $poznamka ->id = $row[0];
            $poznamka ->datumOd = $cursor ->stringToTimestamp($row[1]);
            $poznamka ->datumDo = $cursor ->stringToTimestamp($row[2]);
            $poznamka ->typ = $row[3];
            $poznamka ->barva = $row[5];
            $poznamka ->text = $row[4];
            $poznamka ->poznamka = $row[6];

            if (isset($row[7])) {
                $poznamka ->kodTypuRozdeleni = $row[7];
                $poznamka ->rozdeleni = $row[8];
            }

            $poznamky[] = $poznamka;
        }

        $cursor ->close();

        return $poznamky;
    }


    /**
     * @param Database $database
     * @param int $id
     * @return PoznamkaKeDni
     */
    public static function nactiPoznamku($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT PoznamkaKeDni.Id, PoznamkaKeDni.DatumOd, PoznamkaKeDni.DatumDo, PoznamkaKeDni.Typ, PoznamkaKeDni.Popis,' . // 0..4
            '  TypPoznamkyKeDni.Barva, ' . $database->castWideMemo('PoznamkaKeDni.Poznamka') . ', ' . // 5,6
            '  PoznamkaKeDni.KodTypuRozdeleni, PoznamkaKeDni.Rozdeleni' . // 7,8
            ' FROM PoznamkaKeDni INNER JOIN TypPoznamkyKeDni' .
            '  ON TypPoznamkyKeDni.Kod = PoznamkaKeDni.Typ ' .
            ' WHERE PoznamkaKeDni.Id = ' . $params ->addParameter($id);

        $a = self::nactiPoznamkySQL($database, $sql, $params);

        if ($a) {
            return $a[0];
        }

        return null;
    }


    /**
     * Hromadne nacte poznamky ke dni
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param int $typRozdeleni
     * @param string $rozdeleni
     * @return PoznamkaKeDni[]
     */
    public static function nactiPoznamkyKeDni($database, $datumOd, $datumDo, $typRozdeleni, $rozdeleni)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT PoznamkaKeDni.Id, PoznamkaKeDni.DatumOd, PoznamkaKeDni.DatumDo, PoznamkaKeDni.Typ, PoznamkaKeDni.Popis,' .
            '  TypPoznamkyKeDni.Barva, ' . $database ->castWideMemo('PoznamkaKeDni.Poznamka') .
            ' FROM PoznamkaKeDni INNER JOIN TypPoznamkyKeDni' .
            '  ON TypPoznamkyKeDni.Kod = PoznamkaKeDni.Typ ' .
            (isset($rozdeleni)
                ? ' LEFT OUTER JOIN HierarchieRozdeleni' .
                ' ON HierarchieRozdeleni.TypNadrazene = PoznamkaKeDni.KodTypuRozdeleni' .
                ' AND HierarchieRozdeleni.KodNadrazene = PoznamkaKeDni.Rozdeleni'
                : ''
            ) .
            ' WHERE (TypPoznamkyKeDni.ZobrazitVHlavicce = 1  OR  TypPoznamkyKeDni.ZobrazitVPaticce = 1)' .
            '   AND PoznamkaKeDni.DatumDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '   AND PoznamkaKeDni.DatumOd <= ' . $params ->addParameter($database ->dateToParam($datumDo));

        if (isset($rozdeleni)) {
            $sql .=
                ' AND (' .
                '(HierarchieRozdeleni.TypPodrizene = ' . $params ->addParameter((int)$typRozdeleni) .
                ' AND HierarchieRozdeleni.KodPodrizene = ' . $params ->addParameter($rozdeleni) .
                ') OR PoznamkaKeDni.Rozdeleni IS NULL' .
                ' )';
        }

        $sql .= ' ORDER BY PoznamkaKeDni.DatumOd';

        return self::nactiPoznamkySQL($database, $sql, $params);
    }


    /**
     * Nacte poznamky pro zadanou osobu (podle RozdeleniOsob) v zadanem obdobi.
     * @param Database $database
     * @param int $idpp
     * @param int $datumOd
     * @param int $datumDo
     * @param string[]|string $typy Nepovinny parametr - kod typu nebo seznam typu, ktere se maji nacitat. Pokud neni zadan, nacitaji se vsechny.
     * @return PoznamkaKeDni[]
     */
    public static function nactiPoznamkyProOsobu($database, $idpp, $datumOd, $datumDo, $typy = null)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT PoznamkaKeDni.Id, PoznamkaKeDni.DatumOd, PoznamkaKeDni.DatumDo, PoznamkaKeDni.Typ, PoznamkaKeDni.Popis, ' .
            '  TypPoznamkyKeDni.Barva, ' . $database ->castWideMemo('PoznamkaKeDni.Poznamka') .
            ' FROM PoznamkaKeDni, TypPoznamkyKeDni, RozdeleniOsob, PracovniPomer ' .
            ' WHERE TypPoznamkyKeDni.Kod = PoznamkaKeDni.Typ ' .
            // poznamky v zadanem obdobi
            '  AND PoznamkaKeDni.DatumDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '  AND PoznamkaKeDni.DatumOd <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            // prirazene k rozdeleni zadane osoby
            '  AND PoznamkaKeDni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni' .
            '  AND PoznamkaKeDni.Rozdeleni = RozdeleniOsob.Rozdeleni' .
            '  AND PracovniPomer.RC = RozdeleniOsob.RC ' .
            '  AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo ' .
            '  AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu ' .
            '  AND PracovniPomer.Id = ' . $params ->addParameter((int)$idpp) .
            '  AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '  AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)';

        if ($typy) {
            $sql .=
                ' AND (' .
                SQLUtils::getINCondition($database, 'PoznamkaKeDni.Typ', $typy, $params) .
                ')';
        }

        return self::nactiPoznamkySQL($database, $sql, $params);
    }
}
