<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\Cursor;
use Ron\DB\DBException;

class UrovenPredavani
{
    /** @var string     Vstupni uroven predavaciho procesu */
    public $pocatecniUroven;
    /** @var int */
    public $poradi;
    /** @var string */
    public $kodUrovne;
    /** @var int        Priznak, ze se na teto urovni nema kontrolovat pravo hierarchie rozdeleni */
    public $nekontrolovatHierarchii;
    /** @var int        Priznak, ze na teto urovni uzivatel muze editovat jen sve vlastni formulare */
    public $vlastni;


    /**
     * Vraci query s prvnimi urovnemi predavani zadaneho procesu, na ktere ma aktualni uzivatel pravo
     * @param Database $database
     * @param string $proces
     * @return Cursor
     * @throws DBException
     */
    private static function queryVstupniUrovnePredavani($database, $proces): Cursor
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT PoradiPredavani.PocatecniUroven, PoradiPredavani.Poradi, PoradiPredavani.KodUrovne, PoradiPredavani.NekontrolovatHierarchii, PoradiPredavani.Vlastni' .
            ' FROM RozdeleniOsob, PMProUrovne, PoradiPredavani ' .
            (User::jeZastup() ? ', ZastupProPredavaciProces, VybraneRoleProZastup, ZastupUzivatele' : '') .  // uzivatel je prihlasen "jako" => proces musi byt predan

            // filtr podle pracovnich mist aktualniho uzivatele
            ' WHERE RozdeleniOsob.RC = ' . $params->addParameter(User::getRC()) .
            '  AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter(time()) .
            '  AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter(time()) . '  OR  RozdeleniOsob.PlatnostDo IS NULL) ' .
            '  AND RozdeleniOsob.KodTypuRozdeleni = PMProUrovne.KodTypuRozdeleni ' .
            '  AND RozdeleniOsob.Rozdeleni = PMProUrovne.KodRozdeleni ' .

            // filtr pro "PoradiPredavani"
            '  AND PoradiPredavani.KodUrovne = PMProUrovne.KodUrovne ' .
            '  AND PoradiPredavani.KodProcesu = ' . $params->addParameter($proces);

        // zastup - jen predavaci procesy, ktere jsou uzivateli predane
        if (User::jeZastup()) {
            $sql .=
                '  AND PoradiPredavani.KodProcesu = ZastupProPredavaciProces.KodProcesu' .
                '  AND ZastupProPredavaciProces.KodRole = VybraneRoleProZastup.KodRole' .
                '  AND VybraneRoleProZastup.IdZastupu = ZastupUzivatele.Id' .
                '  AND ZastupUzivatele.Zastupovany = ' . $params ->addParameter(User::getRC()) .
                '  AND ZastupUzivatele.Zastupujici = ' . $params ->addParameter(User::getRealUserRC()) .
                '  AND ZastupUzivatele.Potvrzeni <= 1' . // jen stav "nepotvrzeno" nebo "potvrzeno" (vynecha zamitnuto a zruseno)
                '  AND ZastupUzivatele.PlatnostOd <= ' . $params ->addDateParameter(time()) .
                '  AND (ZastupUzivatele.PlatnostDo >= ' . $params ->addDateParameter(time()) . ' OR ZastupUzivatele.PlatnostDo IS NULL)';
        }

        // jen prvni uroven - tj. neexistuje predchozi
        // POZOR - pokud je tato cast dotazu pred predchozim kodem (if jeZastup ...), tak to nefunguje!
        $sql .=
            '  AND NOT EXISTS ( ' .
            '    SELECT * FROM PoradiPredavani P2 ' .
            '    WHERE P2.KodProcesu = PoradiPredavani.KodProcesu' .
            '    AND P2.PocatecniUroven = PoradiPredavani.PocatecniUroven ' .
            '    AND P2.Poradi < PoradiPredavani.Poradi ' .
            '  ) ';

        return $database ->getSqlResult($sql, $params);
    }


    /**
     * Vraci seznam objektu UrovenPredavani - pocatecni urovne, na ktere ma uzivatel pravo
     * @param Database $database
     * @param string $proces
     * @return UrovenPredavani[]
     * @throws DBException
     */
    static function nactiVstupniUrovnePredavani($database, $proces): array
    {
        $result = [];

        $cursor = self::queryVstupniUrovnePredavani($database, $proces);

        while ($row = $cursor ->fetch()) {
            $uroven = new UrovenPredavani();
            $uroven->pocatecniUroven = $row[0];
            $uroven->poradi = (int)$row[1];
            $uroven->kodUrovne = $row[2];
            $uroven->nekontrolovatHierarchii = (int)$row[3];
            $uroven->vlastni = (int)$row[4];

            $result[] = $uroven;
        }

        $cursor ->close();

        return $result;
    }


    /**
     * Vraci filtr vstupnich urovni, na ktere ma uzivatel pravo
     * @param Database $database
     * @param string $proces
     * @param string $jmenoPole Toto pole se bude filtrovat
     * @return string
     * @throws DBException
     */
    static function filtrVstupniUrovnePredavani($database, $proces, $jmenoPole): string
    {
        $list = self::nactiVstupniUrovnePredavani($database, $proces);

        $result = '';

        foreach ($list as $item) {
            $result .=
                ($result ? ' OR ' : '') .
                "$jmenoPole = '" . $database ->addSlashes($item->pocatecniUroven) . "'";
        }

        return $result;
    }


    /**
     * Vraci vstupni uroven procesu podle aktualniho uzivatele a zadane osoby (bud RC nebo PP)
     * @param Database $database
     * @param string $predavaciProces
     * @param string $rc
     * @param ?string $osobniCislo
     * @param ?int $datumNastupu
     * @return UrovenPredavani
     * @throws \Exception
     */
    static function vstupniUrovenProcesuPodleOsoby($database, $predavaciProces, $rc, $osobniCislo = null, $datumNastupu = null): ?UrovenPredavani
    {
        $params = $database ->createParameters();

        // (osoba) PM_Osoba  <=  PMU_Osoba  <=  PocatecniUroven-PORADI-KodUrovne  =>  PMU_uzivatel  =>  PM_uzivatel

        $sql =
            'SELECT PORADI.PocatecniUroven, PORADI.Poradi, PORADI.KodUrovne, PORADI.NekontrolovatHierarchii, PORADI.Vlastni' .
            '  FROM RozdeleniOsob PM_osoba, PMProUrovne PMU_osoba,' .
            (User::jeZastup() ? ' ZastupProPredavaciProces, VybraneRoleProZastup, ZastupUzivatele,' : '') .  // uzivatel je prihlasen "jako" => proces musi byt predan
            '       PoradiPredavani PORADI,' .
            '       PMProUrovne PMU_uzivatel, RozdeleniOsob PM_uzivatel' .
            ' WHERE PM_osoba.KodTypuRozdeleni = PMU_osoba.KodTypuRozdeleni' .
            '   AND PM_osoba.Rozdeleni = PMU_osoba.KodRozdeleni' .
            '   AND PORADI.PocatecniUroven = PMU_osoba.KodUrovne' .
            '   AND PMU_uzivatel.KodUrovne = PORADI.KodUrovne' .
            // jen aktualni pracovni misto uzivatele
            '   AND PM_uzivatel.RC = ' . $params->addParameter(User::getRC()) .
            '   AND PM_uzivatel.KodTypuRozdeleni = PMU_uzivatel.KodTypuRozdeleni' .
            '   AND PM_uzivatel.Rozdeleni = PMU_uzivatel.KodRozdeleni' .
            '   AND PM_uzivatel.PlatnostOd <= ' . $params->addDateParameter(time()) .
            '   AND (PM_uzivatel.PlatnostDo >= ' . $params->addDateParameter(time()) . '  OR  PM_uzivatel.PlatnostDo IS NULL)' .
            // proces podle aktualniho XML
            '   AND PORADI.KodProcesu = ' . $params->addParameter($predavaciProces) .
            // vybrany zamestnanec/pracovni pomer
            '   AND PM_osoba.RC = ' . $params->addParameter($rc);

        if ($osobniCislo && $datumNastupu) {
            $sql .=
                ' AND PM_osoba.OsobniCislo = ' . $params ->addParameter($osobniCislo) .
                ' AND PM_osoba.DatumNastupu = ' . $params ->addDateParameter($datumNastupu);
        }

        // pro budouci nastupy filtruje PM_osoba k datu nastupu
        $sql .=
            '   AND PM_osoba.PlatnostOd <= ' . $params->addDateParameter((int)$datumNastupu <= time() ? time() : $datumNastupu) .
            '   AND (PM_osoba.PlatnostDo >= ' . $params->addDateParameter((int)$datumNastupu <= time() ? time() : $datumNastupu) . '  OR  PM_osoba.PlatnostDo IS NULL)';

        if (User::jeZastup()) {
            // Uzivatel je jen zastup - doplni filtr na povolene procesy
            $sql .=
                '  AND PORADI.KodProcesu = ZastupProPredavaciProces.KodProcesu' .
                '  AND ZastupProPredavaciProces.KodRole = VybraneRoleProZastup.KodRole' .
                '  AND VybraneRoleProZastup.IdZastupu = ZastupUzivatele.Id' .
                '  AND ZastupUzivatele.Zastupovany = ' . $params ->addParameter(User::getRC()) .
                '  AND ZastupUzivatele.Zastupujici = ' . $params ->addParameter(User::getRealUserRC()) .
                '  AND ZastupUzivatele.Potvrzeni <= 1' . // jen stav "nepotvrzeno" nebo "potvrzeno" (vynecha zamitnuto a zruseno)
                '  AND ZastupUzivatele.PlatnostOd <= ' . $params ->addDateParameter(time()) .
                '  AND (ZastupUzivatele.PlatnostDo >= ' . $params ->addDateParameter(time()) . ' OR ZastupUzivatele.PlatnostDo IS NULL)';
        }


        // jen prvni uroven - tj. neexistuje predchozi
        $sql .=
            ' AND NOT EXISTS ( ' .
            '  SELECT * FROM PoradiPredavani P2 ' .
            '  WHERE P2.KodProcesu = PORADI.KodProcesu ' .
            '    AND P2.PocatecniUroven = PORADI.PocatecniUroven ' .
            '    AND P2.Poradi < PORADI.Poradi ' .
            ' ) ';

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            return null;
        }

        $result = new UrovenPredavani();
        $result ->pocatecniUroven = $row[0];
        $result ->poradi = $row[1];
        $result ->kodUrovne = $row[2];
        $result ->nekontrolovatHierarchii = $row[3];
        $result ->vlastni = $row[4];

        return $result;
    }


    /**
     * Vraci true, pokud je v zadanem procesu i nizsi uroven predavani
     * @param Database $database
     * @param string $kodProcesu
     * @param string $pocatecniUroven
     * @param int $poradiPredavani
     * @return bool
     * @throws DBException
     */
    public static function existujeNizsiUrovenPredavani($database, $kodProcesu, $pocatecniUroven, $poradiPredavani): bool
    {
        if (!$kodProcesu  ||  !$pocatecniUroven  ||  !$poradiPredavani) {
            return false;
        }

        $params = $database ->createParameters();
        $sql =
            'SELECT Poradi FROM PoradiPredavani ' .
            ' WHERE PoradiPredavani.KodProcesu = ' . $params ->addParameter($kodProcesu) .
            '   AND PoradiPredavani.PocatecniUroven = ' . $params ->addParameter($pocatecniUroven) .
            '   AND PoradiPredavani.Poradi < ' . $params ->addParameter((int)$poradiPredavani);

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return !!$row;
    }


    /**
     * Vraci true pokud v zadanem predavacim procesu existuje vyssi uroven predavani
     * @param Database $database
     * @param string $kodProcesu
     * @param string $pocatecniUroven
     * @param int $poradiPredavani
     * @return bool
     * @throws DBException
     */
    public static function existujeVyssiUrovenPredavani($database, $kodProcesu, $pocatecniUroven, $poradiPredavani): bool
    {
        if (!$kodProcesu  ||  !$pocatecniUroven  ||  !$poradiPredavani) return false;

        $params = $database ->createParameters();
        $sql =
            'SELECT Poradi FROM PoradiPredavani ' .
            ' WHERE PoradiPredavani.KodProcesu = ' . $params ->addParameter($kodProcesu) .
            '   AND PoradiPredavani.PocatecniUroven = ' . $params ->addParameter($pocatecniUroven) .
            '   AND PoradiPredavani.Poradi > ' . $params ->addParameter((int)$poradiPredavani);

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return !!$row;
    }

}
