<?php
/**
 * stanislav.elbl, 14.02.2022 13:24
 */

namespace Ron\RonPortal\Zamestnanci;
use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\StringUtils;
use RonPortal\Personalistika\PersonUtils;
use RonPortal\Personalistika\RodneCislo;
use RonPortal\Zamestnanci\ZamestnanciUtils;

/**
 * Formularove pole pro nastaveni PlatnostOd u podrizenych zaznamu.
 * Class NovyZamestnanecPlatnostOdField
 * @package RonPortal\Zamestnanci
 */
class NovyZamestnanecPlatnostOdField extends FormField
{
    /** platnost od se nastavi podle datumu nastupu */
    const PLATNOST_DLE_PP = 'datumnastupu';
    /** platnost od se nastavi podle aktualniho data */
    const PLATNOST_TODAY = 'datum';
    /** platnost od bude editovatelna */
    const PLATNOST_EDIT = 'edit';


    /** @var string     Urcuje, podle ceho se nastavi vychozi hodnota */
    public $vychoziNastaveni = self::PLATNOST_DLE_PP;

    /** @var bool Urcuje, jestli se musi vzdy priradit platnost od zacatku mesice */
    public $zacatekMesice = false;

    /**
     * NovyZamestnanecPlatnostOdField constructor.
     * @param string $vychoziNastaveni  Jedna z konstant PLATNOST_...
     * @param bool $zacatekMesice
     */
    public function __construct($vychoziNastaveni = self::PLATNOST_DLE_PP, $zacatekMesice = false)
    {
        parent::__construct($vychoziNastaveni == self::PLATNOST_EDIT
            ? FormField::FIELD_TYPE_DATE
            : FormField::FIELD_TYPE_CONSTANT);

        $this ->vychoziNastaveni = $vychoziNastaveni;
        $this ->zacatekMesice = $zacatekMesice;

        $this ->setDataType('timestamp');

        if ($vychoziNastaveni == self::PLATNOST_TODAY) {
            $this ->setReadOnly(true);
        }
    }


    /**
     * @param int $datumNastupu
     */
    public function nastavPodleDatumuNastupu($datumNastupu)
    {
        //echo "Nastavuji podle nastupu: {$this ->tableName}.{$this ->fieldName} = $datumNastupu <br/>\n";
        if ($this ->fieldType == FormField::FIELD_TYPE_CONSTANT) {
            $datum = $datumNastupu;
            if ($this ->vychoziNastaveni == self::PLATNOST_TODAY) {
                if ($datumNastupu > DateUtils::today()) {
                    $datum = DateUtils::today();
                }
            }

            $this ->defaultValue = $this ->zacatekMesice
                ? DateUtils::firstDayOfMonth($datum)
                : $datum;
        }
    }
}


class NovyZamestnanecRCField extends FormField
{
    public $showReadonly = false;

    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($this ->showReadonly) {
            echo '<input type="hidden" name="' . $this ->fieldName . '" value="' . StringUtils::htmlTextConvert($value) . '"/>';
            echo StringUtils::htmlTextConvert($value);
        } else {
            parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
        }
    }
}


/**
 * Pole pro potvrzeni vadneho RC (spatny format). Zobrazuje zaskrtavatko pro potvrzeni zapisu vadneho RC.
 * Class NovyZamestnanecRCErrorField
 * @package RonPortal\Zamestnanci
 */
class NovyZamestnanecRCErrorField extends FormField
{
    /** @var string     Text chybove hlasky z kontroly RC */
    public $errorMessage;

    /** @var RodneCislo */
    public $analyzaRC = null;

    /**
     * NovyZamestnanecRCErrorField constructor.
     */
    public function __construct()
    {
        parent::__construct(self::FIELD_TYPE_CHECK);
        $this
            ->setFieldName('*RC_Zkontrolovano')
            ->setRowClass('row errortext')
            ->setCaption2('Potvrzuji_spatne_RC');
    }

    public function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($this ->errorMessage) {
            echo '<div class="errorcolor">' . $this ->errorMessage . '</div>';
        }

        if (!$this ->readOnly) {
            parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
        } else {
            echo '<input type="hidden" name="RC_Zkontrolovano" value="1"/>';
        }
    }


    /**
     * Provede analyzu zadaneho RC a podle vysledku nastavi pole formulare
     * @param string $rc
     */
    public function analyzujRC($rc)
    {
        $this ->analyzaRC = PersonUtils::analyzujRC($rc);

        $this ->errorMessage = $this ->analyzaRC ->error;
        $this ->setVisible((bool)$this ->analyzaRC ->error);
    }


    public function lzePokracovat()
    {
        if (!$this ->visible) {
            return true;
        }

        return isset($_POST['RC_Zkontrolovano'])  &&  (int)$_POST['RC_Zkontrolovano'];
    }
}


class NovyZamestnanecOsobniCisloField extends FormField
{
    /** @var Database */
    protected $database;

    /** @var bool */
    protected $insert;

    /** @var string     Kod typu PP, podle ktereho se urci ciselna rada pro vygenerovani noveho osobniho cisla */
    protected $kodTypuPP = 'HPP';

    /**
     * @param Database $database
     * @param bool $insert
     */
    public function __construct($database, $insert = true)
    {
        parent::__construct($insert ? self::FIELD_TYPE_CONSTANT : self::FIELD_TYPE_EDIT);

        $this ->database = $database;
        $this ->insert = $insert;

        $this ->setReadOnly(true);
    }


    protected function loadConfigurationItem($key, $value)
    {
        switch ($key) {
            case 'kodTypuPP' :
                $this ->kodTypuPP = $value;
                break;

            default :
                parent::loadConfigurationItem($key, $value);
        }

        if ($key === 'readOnly') {
            $this ->fieldType = ($this ->insert  &&  $this ->readOnly)
                ? self::FIELD_TYPE_CONSTANT
                : self::FIELD_TYPE_EDIT;
        }
    }


    public function checkRequired($edit)
    {
        if ($this ->readOnly) {
            return true;
        }

        return parent::checkRequired($edit); // TODO: Change the autogenerated stub
    }


    public function getPostValue($utf8)
    {
        if ($this ->fieldType === self::FIELD_TYPE_CONSTANT) {
            // automaticke vygenerovani osobniho cisla
            return ZamestnanciUtils::getNextOsobniCisloTypuPP($this ->database, $this ->kodTypuPP);
        }

        return parent::getPostValue($utf8);
    }
}
